
#include <iostream>
#include <cstdio>
#include <iomanip>
#include <vector>

#include "badDFT2.h"

using namespace std;

void print2D(string name, float* mm, int nx, int ny){
	cout << name << ":" << endl;
  for (int j=0; j<ny; j++){ 
		for (int i=0; i<nx; i++) 
			cout << setw(6) << mm[nx*j + i];
		cout << endl;
	}
	cout << endl;

  return;
}

bool saveField( Ipp32f *ff, int sX, int sY, const char *name ){
	FILE *file = fopen( name, "w" );	// Saves kernels
	if ( file == NULL )  {
    cout << "Couldn't open your file, sorry!" << endl;
    return false;
  }
	for(int i = 0; i<sY; i++){
		for(int j = 0; j<sX; j++)
			fprintf(file, "%2.8f\t", ff[i*sX + j]);
		fprintf(file, "\n");
	}
	fclose(file);

	return true;
}
//---------------------------------------------------------------------
void main1D( const char* filename, int sel, bool verbose ){
	int tMax, nFields;//, sFieldX, sFieldY;
	BAD::badDFT2 d1( filename );	// create a dynamic field object

	//return;
	
	nFields = d1.nFields;				// number of the fields
	tMax = d1.tMax;							// time of simulation

	cout << endl << "Initializing stimuli... ";
	vector<Ipp32f*> stimuli;				// will store stimuli for each field
	stimuli.resize( nFields );
	for(int i=0; i<nFields; i++){
//		int f0size = d1.fieldList[i].sizeF;
		stimuli[i] = ippsMalloc_32ff( d1.fieldList[i].sizeF );
		ippsZero_32ff( stimuli[i], d1.fieldList[i].sizeF );	
	}
//	int f0size = d1.fieldList[0].sizeF;
	d1.fieldList[0].tools->gauss1D( 6.0, 45.0, 5.0, d1.fieldList[0].sizeF );
	ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
								d1.fieldList[0].sizeF );
	cout << "done" << endl;

	saveField( stimuli[0], d1.fieldList[0].sizeF, 1, "stim0.txt" );
	d1.saveKernels( "myKernels.txt" );
	if(verbose){
		FILE *fileU = fopen( "dftUU.txt", "w" );	// Saves kernels
		FILE *fileV = fopen( "dftVV.txt", "w" );	// Saves kernels
		FILE *fileM = fopen( "dftMM.txt", "w" );	// Saves kernels
		switch(sel){
			case 1:
				for(int t = 0; t<tMax; t++){
					for(int i=0; i<d1.fieldList[0].sizeF; i++)
						fprintf(fileU, "%2.8f\t", d1.fieldList[0].data[i]);
					fprintf(fileU, "\n");
					if( t==99 ){
						d1.fieldList[0].tools->gauss1D( 6.0, -10.0, 5.0, 
																					d1.fieldList[0].sizeF );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					//cout << setw(3) << left << setfill(' ') << t+1 << "  ";
					d1.dftStep( stimuli );
				}
				break;
			case 2:
				for(int t = 0; t<tMax; t++){
					for(int i=0; i<d1.fieldList[0].sizeF; i++){
						fprintf(fileU, "%2.8f\t", d1.fieldList[0].data[i]);
						fprintf(fileM, "%2.8f\t", d1.fieldList[1].data[i]);
					}
					fprintf(fileU, "\n");	fprintf(fileM, "\n");
					if( t == 99 ){
						d1.fieldList[0].tools->gauss1D( 6.0, -10.0, 5.0, d1.fieldList[0].sizeF );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
	//				cout << setw(3) << left << setfill(' ') << t+1 << ": ";
					d1.dftStep( stimuli );
	//				cout << endl;
				}
				break;
			case 3:
				for(int t = 0; t<tMax; t++){
					for(int i=0; i<d1.fieldList[0].sizeF; i++){
						fprintf(fileU, "%2.8f\t", d1.fieldList[0].data[i]);
						fprintf(fileV, "%2.8f\t", d1.fieldList[1].data[i]);
					}
					fprintf(fileU, "\n");	fprintf(fileV, "\n");
					if( t == 99 ){
						d1.fieldList[0].tools->gauss1D( 6.0, -10.0, 5.0, d1.fieldList[0].sizeF );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					//cout << setw(3) << right << setfill(' ') << t+1 << ": ";
					d1.dftStep( stimuli );
				}
				break;
			case 4:
				for(int t = 0; t<tMax; t++){
					for(int i=0; i<d1.fieldList[0].sizeF; i++){
						fprintf(fileU, "%2.8f\t", d1.fieldList[0].data[i]);
						fprintf(fileV, "%2.8f\t", d1.fieldList[1].data[i]);
						fprintf(fileM, "%2.8f\t", d1.fieldList[2].data[i]);
					}
					fprintf(fileU, "\n");	fprintf(fileV, "\n");	fprintf(fileM, "\n");
					if( t == 99 ){
						d1.fieldList[0].tools->gauss1D( 6.0, -10.0, 5.0, d1.fieldList[0].sizeF );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					d1.dftStep( stimuli );
				}
				break;
			case 5:
				FILE *file1Da = fopen( "f1Da.txt", "w" );	// Saves kernels
				FILE *file1Db = fopen( "f1Db.txt", "w" );	// Saves kernels
				int sFieldX = d1.fieldList[1].dimList[0].sizeD;
				int sFieldY = d1.fieldList[1].dimList[1].sizeD;
				d1.fieldList[2].tools->gauss1D( 5.0, 10.0, 4.0, sFieldY );
				ippsCopy_32ff( d1.fieldList[2].tools->bufferS, stimuli[2], sFieldY );
				saveField( stimuli[2], sFieldY, 1, "stim2.txt" );
				for(int t = 0; t<tMax; t++){
					for(int i=0; i<d1.fieldList[0].sizeF; i++){
						fprintf(file1Da, "%2.8f\t", d1.fieldList[0].data[i]);
						fprintf(file1Db, "%2.8f\t", d1.fieldList[2].data[i]);
					}
					fprintf(file1Da, "\n");	fprintf(file1Db, "\n");
					if(t==95) 
						saveField( d1.fieldList[1].data, sFieldX, sFieldY, "f2D95.txt");
					if(t==110) 
						saveField( d1.fieldList[1].data, sFieldX, sFieldY, "f2D110.txt");
					if( t == 99 ){
						d1.fieldList[0].tools->gauss1D( 6.0, -10.0, 5.0, 
																					d1.fieldList[0].sizeF );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					//cout << setw(3) << right << setfill(' ') << t+1 << ": ";
					d1.dftStep( stimuli );
				}
				fclose(file1Da);	fclose(file1Db);
				break;
		}
		fclose(fileU);		fclose(fileV);			fclose(fileM);
	}else{
		/* ------------ Main loop, no output ------------ */
		for(int t = 0; t<tMax; t++){
			if((t+1)%10 == 0) 	cout << "." << flush;
			if((t+1)%100 == 0)	cout << " " << t+1 << endl;
//      cout << setw(3) << right << t+1 << ": ";
			if( t == 99 ){
				d1.fieldList[0].tools->gauss1D( 6.0, -10.0, 5.0, d1.fieldList[0].sizeF );
				ippsCopy_32ff( d1.fieldList[0].tools->bufferS, stimuli[0], 
											d1.fieldList[0].sizeF );
			}
			d1.dftStep( stimuli );
//      cout << endl;
		}
	}

	for(int i=0; i<nFields; i++)
		ippsFree ( stimuli[i] );

	return;
}

void main2D( const char* filename, int sel, bool verbose ){
	int tMax, sFieldX, sFieldY, nFields;
	BAD::badDFT2 d1( filename );	// create a dynamic field object
	
	sFieldX = d1.fieldList[0].dimList[0].sizeD;	// size of the field (X dimension)
	sFieldY = d1.fieldList[0].dimList[1].sizeD;	// size of the field (Y dimension)
	nFields = d1.nFields;				// number of the fields
	tMax = d1.tMax;							// time of simulation

	cout << endl << "Initializing stimuli... ";
	vector<Ipp32f*> stimuli;				// will store stimuli for each field
	stimuli.resize( nFields );
	for(int i=0; i<nFields; i++){
		stimuli[i] = ippsMalloc_32ff( d1.fieldList[i].sizeF );
		ippsZero_32ff( stimuli[i], d1.fieldList[i].sizeF );	
	}
/* ------- create a noisy 2D input ------
	unsigned int mySeed = 0;
	ippsRandGauss_Direct_32ff( stimuli[0], d1.fieldList[0].sizeF, 0, 1, &mySeed );
	ippsMulC_32ff_I( 5.0, stimuli[0], d1.fieldList[0].sizeF ); 
*/
	d1.fieldList[0].tools->gauss2D( 5.0, 30.0, 5.0, -10.0, 5.0 );
	ippsCopy_32ff( d1.fieldList[0].tools->bufferM, stimuli[0], 
								d1.fieldList[0].sizeF );
	cout << "done" << endl;
	
//	saveField( stimuli[0], sFieldX, sFieldY, "stim0.txt" );
	d1.saveKernels( "myKernels.txt" );
	cout << endl << "Starting the simulation ..." << endl;
	if(verbose){
		switch (sel){
			case 6:
				for(int t = 0; t<tMax; t++){
					if(t==95) saveField( d1.fieldList[0].data, sFieldX, sFieldY, "f2D95.txt");
					if(t==110) saveField( d1.fieldList[0].data, sFieldX, sFieldY, "f2D110.txt");
					if( t==99 ){
						d1.fieldList[0].tools->gauss2D( 5.0, -10.0, 5.0, 20.0, 5.0 );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferM, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					d1.dftStep( stimuli );
				}
				break;
			case 7:
				for(int t = 0; t<tMax; t++){
					if(t==95){
						saveField( d1.fieldList[0].data, sFieldX, sFieldY, "f2D95.txt");
						saveField( d1.fieldList[1].data, sFieldX, sFieldY, "f2D95m.txt");
					}
					if(t==110){
						saveField( d1.fieldList[0].data, sFieldX, sFieldY, "f2D110.txt");
						saveField( d1.fieldList[1].data, sFieldX, sFieldY, "f2D110m.txt");
					}
					if( t==99 ){
						d1.fieldList[0].tools->gauss2D( 5.0, -10.0, 5.0, 20.0, 5.0 );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferM, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					d1.dftStep( stimuli );
				}
				break;
			case 8:
				FILE *fileU = fopen( "f1D.txt", "w" );	// Saves kernels
				for(int t = 0; t<tMax; t++){
					//cout << setw(3) << right << t+1 << ": ";
					for(int i=0; i<d1.fieldList[1].sizeF; i++)
						fprintf(fileU, "%2.8f\t", d1.fieldList[1].data[i]);
					fprintf(fileU, "\n");
					if(t==95) saveField( d1.fieldList[0].data, sFieldX, sFieldY, "f2D95.txt");
					if(t==110) saveField( d1.fieldList[0].data, sFieldX, sFieldY, "f2D110.txt");
					if( t==99 ){
						d1.fieldList[0].tools->gauss2D( 5.0, -10.0, 5.0, 20.0, 5.0 );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferM, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					d1.dftStep( stimuli );
					//cout << endl;
				}
				fclose(fileU);
				break;
		}
	}else{
		for(int t = 0; t<tMax; t++){
			if((t+1)%10 == 0) 	cout << "." << flush;
			if((t+1)%100 == 0)	cout << " " << t+1 << endl;
			//cout << setw(3) << right << t+1 << ": ";
			if( t == 99 ){
				d1.fieldList[0].tools->gauss2D( 5.0, -10.0, 5.0, 20.0, 5.0 );
				ippsCopy_32ff( d1.fieldList[0].tools->bufferM, stimuli[0], 
											d1.fieldList[0].sizeF );
			}
			d1.dftStep( stimuli );
			//cout << endl;
		}
	}

	for(int i=0; i<nFields; i++)
		ippsFree ( stimuli[i] );

	return;
}

void main3D( const char* filename, int sel, bool verbose ){
	int tMax, sFieldX, sFieldY, sFieldZ, nFields;
	BAD::badDFT2 d1( filename );	// create a dynamic field object

	sFieldX = d1.fieldList[0].dimList[0].sizeD;	// size of the field (X dimension)
	sFieldY = d1.fieldList[0].dimList[1].sizeD;	// size of the field (Y dimension)
	sFieldZ = d1.fieldList[0].dimList[2].sizeD;	// size of the field (Z dimension)
	nFields = d1.nFields;				// number of the fields
	tMax = d1.tMax;							// time of simulation

	cout << endl << "Initializing stimuli... ";
	vector<Ipp32f*> stimuli;				// will store stimuli for each field
	stimuli.resize( nFields );
	for(int i=0; i<nFields; i++){
		stimuli[i] = ippsMalloc_32ff( d1.fieldList[i].sizeF );
		ippsZero_32ff( stimuli[i], d1.fieldList[i].sizeF );	
	}
/* ------- create a noisy 2D input ------
	unsigned int mySeed = 0;
	ippsRandGauss_Direct_32ff( stimuli[0], d1.fieldList[0].sizeF, 0, 1, &mySeed );
	ippsMulC_32ff_I( 5.0, stimuli[0], d1.fieldList[0].sizeF ); 
*/
	d1.fieldList[0].tools->gauss3D( 6.0, 30.0, 5.0, -10.0, 5.0, -20.0, 4.0 );
	ippsCopy_32ff( d1.fieldList[0].tools->bufferL, stimuli[0], 
								d1.fieldList[0].sizeF );
	cout << "done" << endl;
	
//	saveField( stimuli[0], sFieldX, sFieldY, "stim0.txt" );
	d1.saveKernels( "myKernels.txt" );
	cout << endl << "Starting the simulation..." << endl;
	if(verbose){
		switch (sel){
			case 9:
				for(int t = 0; t<tMax; t++){
					if(t==49){
						saveField( d1.fieldList[0].data + sFieldX*sFieldY*70, 
															 sFieldX, sFieldY, "f3D70.txt");
						saveField( d1.fieldList[0].data + sFieldX*sFieldY*57, 
															 sFieldX, sFieldY, "f3D57.txt");
					
					}
					if(t==109) saveField( d1.fieldList[0].data + sFieldX*sFieldY*70, 
															 sFieldX, sFieldY, "f3D110.txt");
					if( t==99 ){
						d1.fieldList[0].tools->gauss3D( 6.0, -10.0, 5.0, 20.0, 5.0, 40.0, 4.0 );
						ippsCopy_32ff( d1.fieldList[0].tools->bufferL, stimuli[0], 
													d1.fieldList[0].sizeF );
					}
					d1.dftStep( stimuli );
				}
				break;
			default:
				cout << "I'm doing nothing, what's the matter with u?" << endl;
		}
	}else{
		for(int t = 0; t<tMax; t++){
//			cout << "." << flush;
			if((t+1)%10 == 0) 	cout << "." << flush;
			if((t+1)%100 == 0)	cout << " " << t+1 << endl;
			//cout << setw(3) << right << t+1 << ": ";
			//if( t == 99 ){
				//d1.fieldList[0].tools->gauss3D( 5.0, -10.0, 5.0, 20.0, 5.0, 40.0, 4.0 );
				//ippsCopy_32ff( d1.fieldList[0].tools->bufferL, stimuli[0], 
											//d1.fieldList[0].sizeF );
			//}
			d1.dftStep( stimuli );
			//cout << endl;
		}
	}

	for(int i=0; i<nFields; i++)
		ippsFree ( stimuli[i] );

	return;
}

int main( int argc, char *argv[] ){
	//string str1 = argv[1], str2 = argv[2];
	size_t found;
	int sel;// = 7;
	bool verb = false;
	
	if( argc > 1 ){
		verb = true;
		cout << "With verbose" << endl;
	}
/**/
	cout << "===>>> BADdft2 main tester <<<===" << endl
			 << "Sample cases:" << endl 
			 << "1: 1D[Exc+Inh]\tdft1D1L.cfg" << endl
			 << "2: 1D[Exc+Inh <-> LTM]\tdft1D1Lm.cfg." << endl
			 << "3: 1D[Exc] <-> 1D[Inh]\tdft1D2L.cfg." << endl
			 << "4: 1D[Exc <-> LTM] <-> 1D[Inh]\tdft1D2Lm.cfg." << endl
			 << "5: 1D[Exc+Inh] <-> 2D[Exc+Inh]\tdft1Dto2D.cfg." << endl
			 << "6: 2D[Exc+Inh]\tdft2D1L.cfg." << endl
			 << "7: 2D[Exc+Inh <-> LTM]\tdft2D1Lm.cfg" << endl
			 << "8: 2D[Exc+Inh] <-> 1D[Exc+Inh]\tdft2Dto1D.cfg" << endl
			 << "9: 3D[Exc+Inh]\tdft3D1L.cfg." << endl
			 << "Enter selection: ";
	cin >> sel;
	switch(sel){
		case 1:
			main1D("../confFiles/dft1D1L.cfg", sel, verb );
			break;
		case 2:
			main1D("../confFiles/dft1D1Lm.cfg", sel, verb );
			break;
		case 3:
			main1D("../confFiles/dft1D2L.cfg", sel, verb );
			break;
		case 4:
			main1D("../confFiles/dft1D2Lm.cfg", sel, verb );
			break;
		case 5:
			main1D("../confFiles/dft1Dto2D.cfg", sel, verb );
			break;
		case 6:
			main2D("../confFiles/dft2D1L.cfg", sel, verb );
			break;
		case 7:
			main2D("../confFiles/dft2D1Lm.cfg", sel, verb );
			break;
		case 8:
			main2D("../confFiles/dft2Dto1D.cfg", sel, verb );
			break;
		case 9:
			main3D("../confFiles/dft3D1L.cfg", sel, verb );
			break;
		default:
			cout << "Case not supported, try again!" << endl;
	}
/*	
	found = str1.find_first_of("123");
	dd = atoi(&str1[found]); 
	cout << "Dimensions: " << dd;
	if( dd == 1 ){
		found = str1.find_first_of("12", found+1);
		ll = atoi(&str1[found]);
		found = str1.find_first_of("m", found+1);
		if( int(found) > 0 )
			main1D(argv[1], ll, true, verb );
		else
			main1D(argv[1], ll, false, verb );
	}else if( dd == 2 ){
		found = str1.find_first_of("12", found+1);
		ll = atoi(&str1[found]);
		cout << "\tLayers: " << ll << endl;
		found = str1.find_first_of("m", found+1);
		if( int(found) > 0 )
			main2D(argv[1], ll, true, verb );
		else
			main2D(argv[1], ll, false, verb );	
	}
*/
	cout << "Ciao bello badDFT2" << endl;
	
	return 0;
}

